package com.bonus.material.push.mapper;

import com.bonus.material.basic.domain.BmProject;
import com.bonus.material.push.domain.ConsProjectLeaseMoneyBean;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author : 阮世耀
 * @version : 1.0
 * @PackagePath: com.bonus.material.push.mapper
 * @CreateTime: 2025-01-23  14:40
 * @Description: i皖送费用推送mapper
 */
@Mapper
public interface ConsProjectLeaseMoneyMapper {


    List<ConsProjectLeaseMoneyBean> getRealtionList(ConsProjectLeaseMoneyBean cons);

    List<ConsProjectLeaseMoneyBean> getConsMonthMoney(ConsProjectLeaseMoneyBean o);

    int insertProjectConsMonthInfo(@Param("list") List<ConsProjectLeaseMoneyBean> infos);

    List<ConsProjectLeaseMoneyBean> getScrapMoney(ConsProjectLeaseMoneyBean o);

    int insertProjectOthersMonthInfo(@Param("list") List<ConsProjectLeaseMoneyBean> infos);

    List<ConsProjectLeaseMoneyBean> getLostMoney(ConsProjectLeaseMoneyBean o);

    List<ConsProjectLeaseMoneyBean> getRepairMoney(ConsProjectLeaseMoneyBean o);

    int insertProjectMonthCosts(@Param("list") List<ConsProjectLeaseMoneyBean> total);

    List<BmProject> getIwsProjectList();

    void updateIwsProject(@Param("list") List<BmProject> list);
}
